@@ -1,9 +1,11 @@
 import dataclasses
 import hashlib
 import logging
+import time
 from typing import Optional
 from azure.identity import DefaultAzureCredential
 from azure.storage.blob import BlobClient, ContainerClient
+import tenacity
 import irisml.core
 
 logger = logging.getLogger(__name__)
@@ -43,6 +45,19 @@ class Task(irisml.core.TaskBase):
         if self.config.container_url and not self.config.blob_path:
             raise ValueError("You must specify blob_path if container_url is used.")
 
+        start_time = time.time()
+        downloaded_bytes = self._download()
+        elapsed_time = time.time() - start_time
+        file_hash = hashlib.sha1(downloaded_bytes).hexdigest()
+        logger.info(f"Downloaded {len(downloaded_bytes)} bytes. SHA1 hash: {file_hash}, time: {elapsed_time:.2f}s")
+
+        if self.config.sha1_hash and self.config.sha1_hash != file_hash:
+            raise ValueError(f"The downloaded file has an unexpected hash value. Expected: {self.config.sha1_hash}. Actual: {file_hash}")
+
+        return self.Outputs(downloaded_bytes)
+
+    @tenacity.retry(reraise=True, stop=tenacity.stop_after_attempt(3))
+    def _download(self):
         if self.config.blob_url:
             blob_client = BlobClient.from_blob_url(self.config.blob_url, credential=DefaultAzureCredential())
             downloaded_bytes = blob_client.download_blob().readall()
@@ -50,10 +65,4 @@ class Task(irisml.core.TaskBase):
             container_client = ContainerClient.from_container_url(self.config.container_url, credential=DefaultAzureCredential())
             downloaded_bytes = container_client.download_blob(self.config.blob_path, max_concurrency=8, timeout=300).readall()
 
-        file_hash = hashlib.sha1(downloaded_bytes).hexdigest()
-        logger.info(f"Downloaded {len(downloaded_bytes)} bytes. SHA1 hash: {file_hash}")
-
-        if self.config.sha1_hash and self.config.sha1_hash != file_hash:
-            raise ValueError(f"The downloaded file has an unexpected hash value. Expected: {self.config.sha1_hash}. Actual: {file_hash}")
-
-        return self.Outputs(downloaded_bytes)
+        return downloaded_bytes
